# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2019-2025, by Samuel Williams.

require_relative "event/spawn"
require_relative "event/failure"

module Console
	# Structured event logging.
	module Event
	end
end
