# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2023-2024, by Samuel Williams.

module Console
	# This namespace is reserved for logging adapters provided by other gems.
	module Adapter
	end
end
