# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2025, by Samuel Williams.

require_relative "formatter/progress"
require_relative "formatter/failure"
require_relative "formatter/spawn"

module Console
	module Terminal
		# Formatters for various types of events.
		module Formatter
		end
	end
end
